<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Block\Adminhtml\System\Config;

/**
 * Class AbstractConfig
 * @package FiloBlu\MappConnector\Block\Adminhtml\System\Config
 */
class AbstractConfig extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray
{

    /**
     * Custom Model
     */
    protected $_model;

    /**
     * Resource
     *
     * @var
     */
    protected $_resource;

    /**
     * @var array
     */
    protected $_additionalOptions = [];

    /**
     * AbstractConfig constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ) {
        parent::__construct($context, $data);
    }

    /**
     * Prepare to render
     *
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _prepareToRender()
    {
        $customAttributes = $this->getAttributesToRender();
        array_unshift($customAttributes, ['value' => '', 'label' => __('-- Select an attribute --')]);

        $customAttributesBlock=$this->getLayout()->createBlock('Magento\Framework\View\Element\Html\Select')
            ->setId($this->_getCellInputElementId('<%- _id %>', $this->_id))
            ->setName($this->_getCellInputElementName($this->_id))
            ->setClass('select')
            ->setOptions($customAttributes);

        $this->addColumn($this->_id . '_magento_field', [
            'label' => __('Magento Field'),
            'renderer' => $customAttributesBlock
        ]);
        $this->addColumn($this->_id . '_mapp_field', [
            'label' => __('MAPP Field'),
            'style' => 'width:220px'
        ]);

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
    }

    /**
     * Gets the attribute to render
     *
     * @return array
     */
    protected function getAttributesToRender()
    {
        $customOptions = [];

        if ($this->_type == 'eav') {
            $atts = $this->_model->getAttributes();
            foreach ($atts as $k => $v) {
                if (!in_array($k, $this->_exclude)) {
                    $customOptions[] = ['value' => $k, 'label' => $v->getFrontendLabel() . ' (' . $k . ')'];
                }
            }
        } elseif ($this->_type == 'flat') {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $this->_resource = $objectManager->create('Magento\Framework\App\ResourceConnection');
            $connection = $this->_resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
            $tableName = $this->_model->getCollection()->getMainTable();
            $atts = $connection->describeTable($tableName);
            //$atts = $this->_model->getCollection()->getData();
            foreach ($atts as $k => $v) {
                if (!in_array($k, $this->_exclude)) {
                    $customOptions[] = ['value' => $k, 'label' => str_replace('_', ' ', ucwords($k)) . ' (' . $k . ')'];
                }
            }
        }

        if (count($this->_additionalOptions)) {
            foreach ($this->_additionalOptions as $option) {
                $customOptions[] = $option;
            }
        }

        return $customOptions;
    }
}
