<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\To\Mapp\Resource;

use Exception;
use FiloBlu\MappConnector\Helper\ExceptionHelper;
use FiloBlu\MappConnector\Helper\FileLoggerHelper;
use FiloBlu\MappConnector\Model\Config;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\DataObject as DataObject;

/**
 * Class AbstractResource
 * @package FiloBlu\MappConnector\Model\To\Mapp\Resource
 */
class AbstractResource
{

    /**
     * @var
     */
    protected $_data;

    /**
     * @var
     */
    protected $_list;

    /**
     * @var
     */
    protected $_adapter;

    /**
     * @var
     */
    protected $_defaultadapter;

    /**
     * @var Config
     */
    protected $_config;

    /**
     * @var FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * AbstractResource constructor.
     * @param Config $config
     * @param FileLoggerHelper $fileLogger
     * @throws Exception
     */
    public function __construct(
        Config $config,
        FileLoggerHelper $fileLogger
    ) {
        $this->_fileLogger = $fileLogger;
        $this->_config = $config;
        $this->_init();
    }

    /**
     * Init function
     *
     * @throws Exception
     */
    protected function _init()
    {
        $this->_data = new DataObject();
        if (class_exists($this->_defaultadapter)) {
            $this->_adapter = ObjectManager::getInstance()->create($this->_defaultadapter);
            $this->_adapter->setSystemConfig($this->_config);
            $this->_adapter->setFileLogger($this->_fileLogger);
            $this->_adapter->_init();
        } else {
            ExceptionHelper::throwException('Default Adapter ' . $this->_defaultadapter . ' not exists');
        }
    }

    /**
     * Makes the request to the adapter
     *
     * @param array $params
     * @return mixed
     */
    public function makeRequest($params = [])
    {
        $this->_data->setParams($params);
        $response =  $this->_adapter->makeRequest($this->_data);
        $this->_data->setResponse($response);

        return $response;
    }

    /**
     * Gets data[key] if key set, all data otherwise
     *
     * @param string $key
     * @return mixed
     */
    public function getData($key = '')
    {
        if (!empty($key)) {
            return $this->_data['$key'];
        } else {
            return $this->_data;
        }
    }

    /**
     * Sets Data
     *
     * @param $data
     */
    public function setData($data)
    {
        $this->_data = $data;
    }

    /**
     * Gets the Adapter
     *
     * @return mixed
     */
    public function getAdapter()
    {
        return $this->_adapter;
    }

    /**
     * Sets the Adapter
     *
     * @param $adapter
     */
    public function setAdapter($adapter)
    {
        $this->_adapter = $adapter;
    }

    /**
     * Check if mail domain is excluded
     *
     * @param $email
     * @return bool
     * @throws Exception
     */
    public function _domainIsExcluded($email)
    {
        if ($email != '') {
            try {
                //Get domains to exclude
                $domains_to_exclude = $this->_config->getDomainsToExclude();
                //Get record mail domain
                $mail_domain = explode('@', $email);
                //Check if is excluded
                $check = in_array($mail_domain[1], $domains_to_exclude);
                //Set Log if mail is skipped
                if ($check) {
                    $this->_fileLogger->info('The Job has been skipped because the mail domain (' . $email . ') is excluded by config ' . implode(', ', $domains_to_exclude));
                }
                //Returns true if mail domain is excluded, false otherwise
                return $check;
            } catch (Exception $e) {
                $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
                return true;
            }
        }
        return false;
    }
}
